#include <stdio.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include "lcd.h"

#define NR_ROW	6
#define NR_COL	7

#define MINES		0x0F
#define IS_MINE		0x10	
#define IS_MARKED	0x20
#define IS_UNCOVERED	0x40
#define NR_MINES	5

#define E_INIT_SEED	43


unsigned char table[NR_ROW][NR_COL];
unsigned char EEMEM eepromtable[NR_ROW][NR_COL];
unsigned char EEMEM seed;
unsigned char locx, locy;
unsigned char number;
unsigned char salvat;
unsigned char new_game;

void initialize_table()
{
	unsigned char i, j;  
	for(i = 0; i < NR_ROW; i++)
   		for(j = 0; j < NR_COL; j++)
        		table[i][j] = 0;   
}  

//intializare cu bombe

void initialize_with_mines(unsigned char nr)
{
	unsigned char l = 0, r, c;
	while(l < nr)          
	{              
		r = rand() % NR_ROW;
		c = rand() % NR_COL;
		if ((table[r][c] & IS_MINE) == 0)             
		{ 
			table[r][c] |= IS_MINE;
			l ++;
		}
	}
	
}  

//numar cate bombe sunt in jur  

unsigned char number_of_mines(unsigned char i, unsigned char j)
{
  	unsigned char nr = 0;
	char i1, j1;
         
	for(i1 = -1; i1 <= 1; i1 ++)    
		for(j1 = -1; j1 <= 1; j1 ++)
			if(i1 + i>= 0 && i1 + i <= NR_ROW - 1 && j1 + j >= 0 
			&& j1 + j <= NR_COL - 1 && (i1 != 0 || j1 != 0)) 
				if (table[i + i1][j + j1] & IS_MINE)
					nr ++;

                
  	return nr;
}

//pentru fiecare casuta, calculez numarul de mine din jurul sau

void nr_mines_table()
{
	unsigned char i = 0, j = 0;  
   
   	for (i = 0; i < NR_ROW; i ++)
   		for(j = 0; j < NR_COL; j++)
        	      table[i][j] |= number_of_mines(i,j);
}   

//verific daca se va castiga jocul; se apeleaza cand nu mai sunt mine de descoperit

unsigned char verify_end()
{          
	unsigned char i, j;
                       
   	for(i = 0; i < NR_ROW; i++)              
   		for(j = 0; j < NR_COL; j++)                                                   
            		if ((table[i][j] & IS_MARKED) && ((table[i][j] & IS_MINE) == 0))
            			return -1;
   	return 0;
}

//descoperire casuta

void uncover_cell(unsigned char i, unsigned char j)
{	
	char i1, j1;
	
	for(i1 = -1; i1 <= 1; i1 ++)    
		for(j1 = -1; j1 <= 1; j1 ++)
			if(i1 + i>= 0 && i1 + i <= NR_ROW - 1 && j1 + j >= 0 
			&& j1 + j <= NR_COL - 1 && (i1 != 0 || j1 != 0)) 
				if (table[i + i1][j + j1] & IS_MINE)
						return;
		
	for(i1 = -1; i1 <= 1; i1 ++)    
		for(j1 = -1; j1 <= 1; j1 ++)
			if(i1 + i>= 0 && i1 + i <= NR_ROW - 1 && j1 + j >= 0 
			&& j1 + j <= NR_COL - 1 && (i1 != 0 || j1 != 0)) 
				if ((table[i + i1][j + j1] & IS_UNCOVERED) == 0 && (table[i + i1][j + j1] & IS_MARKED) == 0)
				{
					table[i + i1][j + j1] |= IS_UNCOVERED;
					uncovercell(i + i1, j + j1,  table[i + i1][j + j1] & MINES);
					uncover_cell(i + i1, j + j1);
				}
}

//desenare tabela

void draw_table()
{
	lcd_border();
	unsigned char i, j;
	
	for (i = 1; i < NR_ROW; i ++)
		for(j = 1; j < 84; j++)
			lcd_setpixel(j, i * 8);
	for (i = 1; i < 47; i ++)
		for (j = 1; j < NR_COL; j ++)
			lcd_setpixel(j * 12, i);		
}


//afisare tabela la finalul jocului

void show_table()
{
	unsigned char i, j;  
	    
	for(i = 0; i < NR_ROW; i++)
	{
		
		lcd_gotoXY(0, i);	
		for(j = 0; j < NR_COL; j++)
		{
			if ((table[i][j] & IS_MINE) == 0)
				uncovercell(i, j, table[i][j] & MINES);
			else uncovermine(i, j);
		}
	}

	lcd_update();
}

//intializare cu configuratia jocului salvat

void init_saved_game()
{
	lcd_clear();
	draw_table();
	unsigned char i, j;
	for (i = 0; i < NR_ROW; i ++)
		for (j = 0; j < NR_COL; j ++)
		{
			if (table[i][j] & IS_UNCOVERED)
				uncovercell(i, j, table[i][j] & MINES);
			if (table[i][j] & IS_MARKED)
				mark_cell(i, j);
		}
}

int main(void)
{
	unsigned char i, j, aux, ult = 0xFF;
	unsigned char s = 0;

	if (eeprom_read_byte((uint8_t*)E_INIT_SEED) !='T')
	{
		i = 1;
		eeprom_write_byte(&seed, i); 
		eeprom_write_byte((uint8_t*)E_INIT_SEED,'T');
	}
	else
	{
		s = eeprom_read_byte(&seed);
		if (s == 255)
			s = 0;
		else s ++;
		eeprom_write_byte(&seed, s);
	}
	srand(s);
	lcd_init();
	lcd_clear();
	lcd_gotoXY(5, 2);	
	lcd_str("Minesweeper", 11);
	_delay_ms(2000);
	lcd_gotoXY(5, 2);	
	lcd_str("Load game:6", 11);
	_delay_ms(2000);
	lcd_clear();
	draw_table();
	
	dark_cell(locy, locx);
	DDRC = 0X00;
	PORTC = 0xFF;
	initialize_with_mines(NR_MINES);
	nr_mines_table();
	
	
	for(;;)
	{
		aux = PINC & 0x7F;
		
		while ((PINC & 0x7F) != 0x7F) {}
		if (aux != 0x7f && aux != ult)
		{
			//selectare celula + joc nou
			if ( !( aux &(1<<PC5) ) )
			{
				if (new_game)
				{
					new_game = 0;
					s = eeprom_read_byte(&seed);
					if (s == 255)
						s = 0;
					else s ++;
					eeprom_write_byte(&seed, s);
					srand(s);
					lcd_clear();
					draw_table();
					locy = 0; locx = 0;
					dark_cell(locy, locx);
					initialize_table();
					initialize_with_mines(NR_MINES);
					nr_mines_table();
				}	
				else
				{
					
					salvat = 1;
					dark_cell(locy, locx);
					if((table[locy][locx] & IS_UNCOVERED) == 0 && (table[locy][locx] & IS_MARKED) == 0)
					{
						if (table[locy][locx] & IS_MINE)
						{
							show_table();
							_delay_ms(5000);


							lcd_clear();
							
							lcd_gotoXY(0, 2);	
							lcd_str("You've lost!", 12);
							lcd_gotoXY(0, 3);
							lcd_str("New game:5", 10);
							lcd_gotoXY(0, 4);
							lcd_str("Load game:6", 11);
							_delay_ms(2000);
							new_game = 1;
							salvat = 0;
					
						}
						else
						{
							table[locy][locx] |= IS_UNCOVERED;
							uncovercell(locy, locx, table[locy][locx] & MINES);
							uncover_cell(locy, locx);
							dark_cell(locy, locx);
						}
					}
					else dark_cell(locy, locx);
				}
			}
			//salvare/incarcare joc
	
			if ( !( aux &(1<<PC6) ) )
			{
				if (salvat)
				{
					for (i = 0; i < NR_ROW; i ++)
						for (j = 0; j < NR_COL; j ++)
							eeprom_write_byte(&eepromtable[i][j], table[i][j]);
					lcd_clear();
					salvat = 1;
					lcd_gotoXY(5, 2);	
					lcd_str("Game saved", 10);
				}
				else
				{
					for (i = 0; i < NR_ROW; i ++)
						for (j = 0; j < NR_COL; j ++)
							table[i][j] = eeprom_read_byte(&eepromtable[i][j]);
				
					init_saved_game();
					locy = 0; locx = 0;
					dark_cell(locy, locx);
				}
			}
		
			//jos
		
			if ( !( aux &(1<<PC1) ) )
			{
				salvat = 1;
				if (locy < NR_ROW - 1)
				{
					dark_cell(locy, locx);
					locy ++;
					dark_cell(locy, locx);
				}
			}
		
			//sus
		
			if ( !( aux &(1<<PC3) ) )
			{
				salvat = 1;
			
				if (locy > 0)
				{
					dark_cell(locy, locx);
					locy --;
					dark_cell(locy, locx);
				}
			}
			//stanga
			if ( !( aux &(1<<PC2) ) )
			{
				salvat = 1;
				if (locx > 0)
				{
					dark_cell(locy, locx);
					locx --;
					dark_cell(locy, locx);
				}
			}
			//dreapta
		
			if ( !( aux &(1<<PC4) ) )
			{
				salvat = 1;
				if (locx < NR_COL - 1)
				{
					dark_cell(locy, locx);
					locx ++;
					dark_cell(locy, locx);
				}
			}
			//marcat
			if ( !( aux &(1<<PC0) ) )
			{
				salvat = 1;
				dark_cell(locy, locx);
				if((table[locy][locx] & IS_UNCOVERED) == 0 && (table[locy][locx] & IS_MARKED) == 0 
				&& number < NR_MINES)
				{
				
					mark_cell(locy, locx);
					table[locy][locx] |= IS_MARKED;
					number ++;
					if (number == NR_MINES)
						if (verify_end() == 0)
						{
							lcd_clear();
							
							lcd_gotoXY(5, 2);	
							lcd_str("You've won!", 11);
							lcd_gotoXY(0, 3);
							lcd_str("New game:5", 10);
							lcd_gotoXY(0, 4);
							lcd_str("Load game:6", 11);
							_delay_ms(2000);
							
							new_game = 1;
							salvat = 0;
						}		
				} 
				else 
				if((table[locy][locx] & IS_UNCOVERED) == 0 && (table[locy][locx] & IS_MARKED))
				{
				
					unmark_cell(locy, locx);
					table[locy][locx] &= ~IS_MARKED;
					number --;
				}
				dark_cell(locy, locx);
			}
			ult = aux;
		}
		else ult = aux;
		
	}

}

